import React, { useRef, useState, useEffect } from 'react';
import spline from 'cubic-spline';

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };
  return _extends.apply(this, arguments);
}

var treshold = 2;
var Draw = (function (canvas, ctx, xs, ys) {
  if (!canvas || !ctx) return;
  window.requestAnimationFrame(function () {
    ctx.beginPath();
    ctx.rect(0, 0, canvas.width, canvas.height);
    ctx.fillStyle = '#5f5f5f';
    ctx.fill();
    ctx.beginPath();
    ctx.moveTo(0, canvas.height);
    ctx.lineWidth = 2;
    ctx.strokeStyle = '#f1f1f1';
    ctx.fillStyle = '#f1f1f1';
    for (var i = 0; i < canvas.width / treshold; i++) {
      ctx.lineTo(i * treshold / canvas.width * canvas.width, canvas.height - spline(i * treshold / canvas.width, xs, ys) * canvas.height);
    }
    ctx.lineTo(xs[xs.length - 1] * canvas.width, canvas.height - ys[ys.length - 1] * canvas.height);
    ctx.stroke();
    for (var _i = 0; _i < xs.length; _i++) {
      ctx.beginPath();
      ctx.arc(xs[_i] * canvas.width, canvas.height - ys[_i] * canvas.height, 8, 0, 2 * Math.PI, false);
      ctx.fill();
      ctx.stroke();
    }
  });
});

var round = function round(num, dec) {
  if (dec === void 0) {
    dec = 2;
  }
  return parseFloat(num.toFixed(dec));
};
var Apply = (function (originalImageData, canvas, ctx, curves) {
  if (!canvas || !ctx) return;
  window.requestAnimationFrame(function () {
    var curvesData = {
      a: [],
      r: [],
      g: [],
      b: []
    };
    var imageData = new Uint8ClampedArray(originalImageData.data);
    for (var i = 0; i < 256; i++) {
      curvesData.a.push(parseInt(spline(round(i / 256), curves.a.xs, curves.a.ys) * 256));
      curvesData.r.push(parseInt(spline(round(i / 256), curves.r.xs, curves.r.ys) * 256));
      curvesData.g.push(parseInt(spline(round(i / 256), curves.g.xs, curves.g.ys) * 256));
      curvesData.b.push(parseInt(spline(round(i / 256), curves.b.xs, curves.b.ys) * 256));
    }
    for (var _i = 0; _i < imageData.length; _i += 4) {
      imageData[_i] = curvesData.r[imageData[_i]];
      imageData[_i + 1] = curvesData.g[imageData[_i + 1]];
      imageData[_i + 2] = curvesData.b[imageData[_i + 2]];
      imageData[_i] = curvesData.a[imageData[_i]];
      imageData[_i + 1] = curvesData.a[imageData[_i + 1]];
      imageData[_i + 2] = curvesData.a[imageData[_i + 2]];
    }
    ctx.putImageData(new ImageData(imageData, canvas.width, canvas.height), 0, 0);
  });
});

var round$1 = function round(num, dec) {
  if (dec === void 0) {
    dec = 2;
  }
  return parseFloat(num.toFixed(dec));
};
var defaultCurves = {
  a: {
    xs: [0, 1],
    ys: [0, 1]
  },
  r: {
    xs: [0, 1],
    ys: [0, 1]
  },
  g: {
    xs: [0, 1],
    ys: [0, 1]
  },
  b: {
    xs: [0, 1],
    ys: [0, 1]
  }
};
var index = (function (_ref) {
  var _currentChannel = _ref.currentChannel,
    targetCanvas = _ref.targetCanvas;
  var currentChannel = _currentChannel || 'a';
  var canvasRef = useRef(null);
  var ctx = useRef(null);
  var _useState = useState(defaultCurves),
    currentCurves = _useState[0],
    setCurrentCurves = _useState[1];
  var _useState2 = useState(null),
    originalImageData = _useState2[0],
    setOriginalImageData = _useState2[1];
  var x,
    y,
    dragging,
    draggingPont,
    w,
    h,
    pointsToModify,
    dragged = false,
    shift = false;
  useEffect(function () {
    var onKeyDown = function onKeyDown(e) {
      var key = e.keyCode || e.which;
      if (key == 16) shift = true;
    };
    var onKeyUp = function onKeyUp() {
      shift = false;
    };
    document.addEventListener('keydown', onKeyDown);
    document.addEventListener('keyup', onKeyUp);
    return function () {
      document.removeEventListener('keydown', onKeyDown);
      document.removeEventListener('keyup', onKeyUp);
    };
  }, []);
  useEffect(function () {
    if (originalImageData) return;
    var imageData = targetCanvas.getContext('2d').getImageData(0, 0, targetCanvas.width, targetCanvas.height);
    setOriginalImageData(new ImageData(new Uint8ClampedArray(imageData.data), imageData.width, imageData.height));
  }, []);
  useEffect(function () {
    if (!canvasRef.current) return;
    var canvas = canvasRef.current;
    canvas.width = 280;
    canvas.height = 280;
    ctx.current = canvas.getContext('2d');
  }, [canvasRef.current, targetCanvas]);
  useEffect(function () {
    setCurrentCurves(defaultCurves);
  }, [currentChannel]);
  useEffect(function () {
    if (!originalImageData) return;
    Apply(originalImageData, targetCanvas, targetCanvas.getContext('2d'), currentCurves);
  }, [currentCurves]);
  useEffect(function () {
    if (!canvasRef.current || !ctx.current) return;
    Draw(canvasRef.current, ctx.current, currentCurves[currentChannel].xs, currentCurves[currentChannel].ys);
  });
  var removePoint = function removePoint(index) {
    var newPoints = currentCurves;
    if (index === 0 || index >= newPoints[currentChannel].xs.length - 1) return;
    newPoints[currentChannel].xs.splice(index, 1);
    newPoints[currentChannel].ys.splice(index, 1);
    setCurrentCurves(_extends({}, newPoints));
  };
  var changeCurve = function changeCurve(points) {
    var curveToChange = {};
    curveToChange[currentChannel] = points;
    setCurrentCurves(_extends({}, currentCurves, {
      curveToChange: curveToChange
    }));
  };
  var addPoint = function addPoint(x, y) {
    var index = null;
    var newPoints = _extends({}, currentCurves);
    for (var i = 0; i < newPoints[currentChannel].xs.length; i++) {
      if (x > newPoints[currentChannel].xs[i] && x < newPoints[currentChannel].xs[i + 1]) {
        index = i + 1;
        continue;
      }
    }
    newPoints[currentChannel].xs.splice(index, 0, x);
    newPoints[currentChannel].ys.splice(index, 0, y);
    setCurrentCurves(newPoints);
  };
  var _onMouseDown = function onMouseDown(e, canvas, points) {
    dragging = false;
    draggingPont = null;
    pointsToModify = null;
    w = canvas.width;
    h = canvas.height;
    x = e.nativeEvent.offsetX || e.nativeEvent.layerX;
    y = h - (e.nativeEvent.offsetY || e.nativeEvent.layerY);
    for (var i = 0; i < points.xs.length; i++) {
      if (x >= points.xs[i] * w - 6 && x <= points.xs[i] * w + 6 && y >= points.ys[i] * h - 6 && y <= points.ys[i] * h + 6) {
        dragging = true;
        draggingPont = i;
        pointsToModify = Object.assign({}, points);
        continue;
      }
    }
    if (dragging && shift) {
      dragging = false;
      draggingPont = null;
      removePoint(draggingPont + 1);
    }
  };
  var _onMouseMove = function onMouseMove(e, canvas, ctx) {
    if (!dragging || draggingPont == 0 || draggingPont == pointsToModify.xs.length - 1) return;
    dragged = true;
    var mx = e.nativeEvent.offsetX || e.nativeEvent.layerX;
    var my = e.nativeEvent.offsetY || e.nativeEvent.layerY;
    pointsToModify.xs[draggingPont] = mx / w;
    pointsToModify.ys[draggingPont] = (h - my) / h;
    Draw(canvas, ctx, pointsToModify.xs, pointsToModify.ys);
  };
  var onMouseUp = function onMouseUp() {
    dragging = false;
    if (dragged) {
      changeCurve(pointsToModify);
      dragged = false;
    }
  };
  var _onDoubleClick = function onDoubleClick(e, points) {
    var dx = e.nativeEvent.offsetX || e.nativeEvent.layerX;
    var dy = h - (e.nativeEvent.offsetY || e.nativeEvent.layerY);
    if (spline(round$1(dx / w), points.xs, points.ys) * h >= dy - 8 && spline(round$1(dx / w), points.xs, points.ys) * h <= dy + 8) {
      addPoint(round$1(dx / w), round$1(dy / h));
    }
  };
  return React.createElement("div", null, React.createElement("canvas", {
    ref: canvasRef,
    onMouseDown: function onMouseDown(e) {
      return _onMouseDown(e, canvasRef.current, currentCurves[currentChannel]);
    },
    onMouseMove: function onMouseMove(e) {
      return _onMouseMove(e, canvasRef.current, ctx.current);
    },
    onMouseUp: onMouseUp,
    onDoubleClick: function onDoubleClick(e) {
      return _onDoubleClick(e, currentCurves[currentChannel]);
    }
  }));
});

var array256 = function array256() {
  var arr = [];
  for (var i = 0; i < 256; i++) {
    arr[i] = 0;
  }
  return arr;
};
var _c = document.createElement('canvas'),
  _ctx = _c.getContext('2d'),
  data,
  histogramData;
var colors = {
  r: '#d22121',
  g: '#079407',
  b: '#1b1bbb',
  a: '#d6d6d6'
};
var Draw$1 = (function (canvas, ctx, image) {
  if (!canvas || !ctx) return;
  window.requestAnimationFrame(function () {
    ctx.beginPath();
    ctx.rect(0, 0, canvas.width, canvas.height);
    ctx.fillStyle = '#5f5f5f';
    ctx.fill();
    histogramData = {};
    histogramData.a = array256();
    histogramData.r = array256();
    histogramData.g = array256();
    histogramData.b = array256();
    if (typeof image.height === 'number') {
      _c.width = parseInt(image.width / 4);
      _c.height = parseInt(image.height / 4);
      _ctx.drawImage(image, 0, 0, _c.width, _c.height);
    } else {
      _ctx = image.getContext('2d');
    }
    data = _ctx.getImageData(0, 0, _c.width, _c.height).data;
    for (var i = 0; i < data.length; i += 4) {
      var _all = parseInt(data[i] / 3) + parseInt(data[i + 1] / 3) + parseInt(data[i + 2] / 3);
      if (_all > histogramData.a.length - 1) _all = histogramData.a.length - 1;
      histogramData.a[_all]++;
      histogramData.r[data[i]]++;
      histogramData.g[data[i + 1]]++;
      histogramData.b[data[i + 2]]++;
    }
    histogramData.max = parseInt(data.length) * 0.75;
    ctx.lineWidth = 1;
    for (var key in colors) {
      ctx.strokeStyle = colors[key];
      ctx.beginPath();
      ctx.moveTo(0, canvas.height);
      for (var _i = 0; _i < 256; _i++) {
        var hist = histogramData[key][_i] / histogramData.max * 256;
        ctx.lineTo(parseInt(_i / 256 * canvas.width), (1 - hist) * (canvas.height * 0.7) + canvas.height * 0.3);
      }
      ctx.lineTo(canvas.width, canvas.height);
      ctx.stroke();
    }
  });
});

var index$1 = (function (_ref) {
  var targetRef = _ref.targetRef;
  var canvasRef = useRef(null);
  var ctx = useRef(null);
  useEffect(function () {
    if (!canvasRef.current) return;
    var canvas = canvasRef.current;
    canvas.width = 280;
    canvas.height = 140;
    ctx.current = canvas.getContext('2d');
  }, [canvasRef.current]);
  useEffect(function () {
    if (!ctx.current || !canvasRef.current) return;
    Draw$1(canvasRef.current, ctx.current, targetRef);
  }, [ctx.current]);
  return React.createElement("div", null, React.createElement("canvas", {
    ref: canvasRef
  }));
});

export { index as ImageCurvesEditor, index$1 as ImageLevelsHistogram };
//# sourceMappingURL=index.modern.js.map
